@@ -94,14 +94,16 @@ totem_fullscreen_move_popups (TotemFullscreen *fs)
 	
 	GdkScreen              *screen;
 	GdkRectangle            fullscreen_rect;
+	GdkWindow              *window;
 	TotemFullscreenPrivate *priv = fs->priv;
 
 	g_return_if_fail (priv->parent_window != NULL);
 
 	/* Obtain the screen rectangle */
 	screen = gtk_window_get_screen (GTK_WINDOW (priv->parent_window));
+	window = gtk_widget_get_window (priv->parent_window);
 	gdk_screen_get_monitor_geometry (screen,
-					 gdk_screen_get_monitor_at_window (screen, priv->parent_window->window),
+					 gdk_screen_get_monitor_at_window (screen, window),
 					 &fullscreen_rect);
 
 	/* Get the popup window sizes */
@@ -256,7 +258,7 @@ totem_fullscreen_set_cursor (TotemFullscreen *fs, gboolean state)
 static gboolean
 totem_fullscreen_is_volume_popup_visible (TotemFullscreen *fs)
 {
-	return GTK_WIDGET_VISIBLE (gtk_scale_button_get_popup (GTK_SCALE_BUTTON (fs->volume)));
+	return gtk_widget_get_visible (gtk_scale_button_get_popup (GTK_SCALE_BUTTON (fs->volume)));
 }
 
 static void
@@ -429,7 +431,7 @@ totem_fullscreen_parent_window_notify (GtkWidget *parent_window,
 	popup = gtk_scale_button_get_popup (GTK_SCALE_BUTTON (fs->volume));
 	if (parent_window == fs->priv->parent_window &&
 	    gtk_window_is_active (GTK_WINDOW (parent_window)) == FALSE &&
-	    GTK_WIDGET_VISIBLE (popup) == FALSE) {
+	    gtk_widget_get_visible (popup) == FALSE) {
 		totem_fullscreen_force_popup_hide (fs);
 		totem_fullscreen_set_cursor (fs, TRUE);
 	} else {
