@@ -40,11 +40,9 @@
 #include "totem-open-location.h"
 #include "totem-interface.h"
 
-static void totem_open_location_dispose		(GObject *object);
-
 struct TotemOpenLocationPrivate
 {
-	GtkBuilder *xml;
+	GtkWidget *uri_container;
 	GtkEntry *uri_entry;
 };
 
@@ -52,35 +50,32 @@ struct TotemOpenLocationPrivate
 
 G_DEFINE_TYPE (TotemOpenLocation, totem_open_location, GTK_TYPE_DIALOG)
 
+/* GtkBuilder callbacks */
+G_MODULE_EXPORT void uri_entry_changed_cb (GtkEditable *entry, GtkDialog *dialog);
+
 static void
 totem_open_location_class_init (TotemOpenLocationClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
 	g_type_class_add_private (klass, sizeof (TotemOpenLocationPrivate));
-	object_class->dispose = totem_open_location_dispose;
 }
 
 static void
 totem_open_location_init (TotemOpenLocation *self)
 {
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, TOTEM_TYPE_OPEN_LOCATION, TotemOpenLocationPrivate);
+	GtkBuilder *builder;
 
-	self->priv->xml = totem_interface_load ("uri.ui", FALSE, NULL, self);
-	self->priv->uri_entry = GTK_ENTRY (gtk_builder_get_object (self->priv->xml, "uri"));
-}
+	self->priv = TOTEM_OPEN_LOCATION_GET_PRIVATE (self);
+	builder = totem_interface_load ("uri.ui", FALSE, NULL, self);
 
-static void
-totem_open_location_dispose (GObject *object)
-{
-	TotemOpenLocationPrivate *priv = TOTEM_OPEN_LOCATION_GET_PRIVATE (object);
+	if (builder == NULL)
+		return;
 
-	if (priv->xml != NULL) {
-		g_object_unref (priv->xml);
-		priv->xml = NULL;
-	}
+	self->priv->uri_container = GTK_WIDGET (gtk_builder_get_object (builder, "open_uri_dialog_content"));
+	g_object_ref (self->priv->uri_container);
 
-	G_OBJECT_CLASS (totem_open_location_parent_class)->dispose (object);
+	self->priv->uri_entry = GTK_ENTRY (gtk_builder_get_object (builder, "uri"));
+
+	g_object_unref (builder);
 }
 
 static gboolean
@@ -89,6 +84,9 @@ totem_open_location_match (GtkEntryCompletion *completion, const gchar *key, Gtk
 	/* Substring-match key against URI */
 	char *uri, *match;
 
+	g_return_val_if_fail (GTK_IS_TREE_MODEL (user_data), FALSE);
+	g_return_val_if_fail (iter != NULL, FALSE);
+
 	g_return_val_if_fail (key != NULL, FALSE);
 	gtk_tree_model_get (user_data, iter, 0, &uri, -1);
 	g_return_val_if_fail (uri != NULL, FALSE);
@@ -114,6 +112,8 @@ totem_open_location_get_uri (TotemOpenLocation *open_location)
 {
 	char *uri;
 
+	g_return_val_if_fail (TOTEM_IS_OPEN_LOCATION (open_location), NULL);
+
 	uri = g_strdup (gtk_entry_get_text (open_location->priv->uri_entry));
 
 	if (strcmp (uri, "") == 0)
@@ -136,6 +136,8 @@ totem_open_location_set_from_clipboard (TotemOpenLocation *open_location)
 	GtkClipboard *clipboard;
 	gchar *clipboard_content;
 
+	g_return_val_if_fail (TOTEM_IS_OPEN_LOCATION (open_location), NULL);
+
 	/* Initialize the clipboard and get its content */
 	clipboard = gtk_clipboard_get_for_display (gtk_widget_get_display (GTK_WIDGET (open_location)), GDK_SELECTION_CLIPBOARD);
 	clipboard_content = gtk_clipboard_wait_for_text (clipboard);
@@ -151,19 +153,25 @@ totem_open_location_set_from_clipboard (TotemOpenLocation *open_location)
 	return NULL;
 }
 
-GtkWidget*
-totem_open_location_new (Totem *totem)
+void
+uri_entry_changed_cb (GtkEditable *entry, GtkDialog *dialog)
+{
+	gboolean sensitive = (gtk_entry_get_text_length (GTK_ENTRY (entry)) > 0);
+	gtk_dialog_set_response_sensitive (dialog, GTK_RESPONSE_OK, sensitive);
+}
+
+GtkWidget *
+totem_open_location_new (void)
 {
 	TotemOpenLocation *open_location;
 	char *clipboard_location;
 	GtkEntryCompletion *completion;
 	GtkTreeModel *model;
 	GList *recent_items, *streams_recent_items = NULL;
-	GtkWidget *container;
 
 	open_location = TOTEM_OPEN_LOCATION (g_object_new (TOTEM_TYPE_OPEN_LOCATION, NULL));
 
-	if (open_location->priv->xml == NULL) {
+	if (open_location->priv->uri_container == NULL) {
 		g_object_unref (open_location);
 		return NULL;
 	}
@@ -174,12 +182,10 @@ totem_open_location_new (Totem *totem)
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_OPEN, GTK_RESPONSE_OK,
 			NULL);
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (open_location), GTK_RESPONSE_OK, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (open_location), 5);
 	gtk_dialog_set_default_response (GTK_DIALOG (open_location), GTK_RESPONSE_OK);
 
-	g_signal_connect (G_OBJECT (open_location), "delete-event",
-			G_CALLBACK (gtk_widget_destroy), open_location);
-
 	/* Get item from clipboard to fill GtkEntry */
 	clipboard_location = totem_open_location_set_from_clipboard (open_location);
 	if (clipboard_location != NULL && strcmp (clipboard_location, "") != 0)
@@ -229,10 +235,8 @@ totem_open_location_new (Totem *totem)
 	gtk_entry_completion_set_text_column (completion, 0);
 	gtk_entry_completion_set_match_func (completion, (GtkEntryCompletionMatchFunc) totem_open_location_match, model, NULL);
 
-	container = GTK_WIDGET (gtk_builder_get_object (open_location->priv->xml,
-				"open_uri_dialog_content"));
 	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (open_location))),
-				container,
+				open_location->priv->uri_container,
 				TRUE,       /* expand */
 				TRUE,       /* fill */
 				0);         /* padding */
